--- loncom/xml/lonxml.pm	2007/10/16 23:20:05	1.465
+++ loncom/xml/lonxml.pm	2008/03/21 18:36:48	1.475
@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # XML Parser Module 
 #
-# $Id: lonxml.pm,v 1.465 2007/10/16 23:20:05 albertel Exp $
+# $Id: lonxml.pm,v 1.475 2008/03/21 18:36:48 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -377,8 +377,12 @@ sub xmlparse {
      $finaloutput .= join('',@script_var_displays);
      undef(@script_var_displays);
  }
-
+ &init_state();
  if ($env{'form.return_only_error_and_warning_counts'}) {
+     if ($env{'request.filename'}=~/\.(html|htm|xml)$/i) { 
+        my $error=&verify_html($content_file_string);
+        if ($error) { $errorcount++; }
+     }
      return "$errorcount:$warningcount";
  }
  return $finaloutput;
@@ -659,6 +663,7 @@ sub setup_globals {
   &init_counter();
   &clear_bubble_lines_for_part();
   &init_state();
+  &set_state('target',$target);
   @Apache::lonxml::pwd=();
   @Apache::lonxml::extlinks=();
   @script_var_displays=();
@@ -722,6 +727,7 @@ sub init_safespace {
   $safehole->wrap(\&Apache::chemresponse::chem_standard_order,$safeeval,
 		  '&chem_standard_order');
   $safehole->wrap(\&Apache::response::check_status,$safeeval,'&check_status');
+  $safehole->wrap(\&Apache::response::implicit_multiplication,$safeeval,'&implicit_multiplication');
 
   $safehole->wrap(\&Apache::lonmaxima::maxima_eval,$safeeval,'&maxima_eval');
   $safehole->wrap(\&Apache::lonmaxima::maxima_check,$safeeval,'&maxima_check');
@@ -1066,28 +1072,28 @@ Increments the internal counter environm
 Optional Arguments:
   $increment - amount to increment by (defaults to 1)
                Also 1 if the value is negative or zero.
-  $part_id   - optional part id.. during analysis, this
-               indicates whic part of a problem is being
-               counted.
+  $part_response - A concatenation of the part and response id
+                   identifying exactly what is being 'answered'.
+
 
 =cut
 
 sub increment_counter {
-    my ($increment, $part_id) = @_;
+    my ($increment, $part_response) = @_;
     if (!defined($increment) || $increment le 0) {
 	$increment = 1;
     }
     $Apache::lonxml::counter += $increment;
 
     # If the caller supplied the part_id parameter, 
     # Maintain its counter.. creating if necessary.
 
-    if(defined($part_id)) {
-	if (!defined($Apache::lonxml::counters_per_part{$part_id})) {
-	    $Apache::lonxml::counters_per_part{$part_id} = 0;
+    if (defined($part_response)) {
+	if (!defined($Apache::lonxml::counters_per_part{$part_response})) {
+	    $Apache::lonxml::counters_per_part{$part_response} = 0;
 	}
-	$Apache::lonxml::counters_per_part{$part_id} += $increment;
-	my $new_value = $Apache::lonxml::counters_per_part{$part_id};
+	$Apache::lonxml::counters_per_part{$part_response} += $increment;
+	my $new_value = $Apache::lonxml::counters_per_part{$part_response};
     }
 	
     $Apache::lonxml::counter_changed=1;
@@ -1115,7 +1121,7 @@ sub init_counter {
 }
 
 sub store_counter {
-    &Apache::lonnet::appenv(('form.counter' => $Apache::lonxml::counter));
+    &Apache::lonnet::appenv({'form.counter' => $Apache::lonxml::counter});
     $Apache::lonxml::counter_changed=0;
     return '';
 }
@@ -1136,7 +1142,7 @@ sub store_counter {
 
     sub restore_problem_counter {
 	if (defined($state)) {
-	    &Apache::lonnet::appenv(('form.counter' => $state));
+	    &Apache::lonnet::appenv({'form.counter' => $state});
 	}
     }
     sub get_problem_counter {
@@ -1148,22 +1154,21 @@ sub store_counter {
 
 =pod
 
-=item  bubble_lines_for_part(part_id)
+=item  bubble_lines_for_part(part_response)
 
 Returns the number of lines required to get a response for
-$part_id (this is just $Apache::lonxml::counters_per_part{$part_id}
+$part_response (this is just $Apache::lonxml::counters_per_part{$part_response}
 
 =cut
 
 sub bubble_lines_for_part {
-    my ($part_id) = @_;
+    my ($part_response) = @_;
 
-    if (!defined($Apache::lonxml::counters_per_part{$part_id})) {
+    if (!defined($Apache::lonxml::counters_per_part{$part_response})) {
 	return 0;
     } else {
-	return $Apache::lonxml::counters_per_part{$part_id};
+	return $Apache::lonxml::counters_per_part{$part_response};
     }
-
 }
 
 =pod
@@ -1182,7 +1187,7 @@ sub clear_bubble_lines_for_part {
 
 =pod
 
-=item set_bubble_lines(part_id, value)
+=item set_bubble_lines(part_response, value)
 
 If there is a problem part, that for whatever reason
 requires bubble lines that are not
@@ -1192,9 +1197,9 @@ analysis to set its hash value explicitl
 =cut
 
 sub set_bubble_lines {
-    my ($part_id, $value) = @_;
+    my ($part_response, $value) = @_;
 
-    $Apache::lonxml::counters_per_part{$part_id} = $value;
+    $Apache::lonxml::counters_per_part{$part_response} = $value;
 }
 
 =pod
@@ -1374,7 +1379,7 @@ sub writeallows {
  	        &Apache::lonnet::hreflocation($thisdir,&unescape($_))}=$thisurl;
     }
     @extlinks=();
-    &Apache::lonnet::appenv(%httpref);
+    &Apache::lonnet::appenv(\%httpref);
 }
 
 sub register_ssi {
@@ -1474,11 +1479,26 @@ SIMPLECONTENT
   return $filecontents;
 }
 
+sub verify_html {
+    my ($filecontents)=@_;
+    if ($filecontents!~/(?:\<|\&lt\;)(?:html|xml)[^\<]*(?:\>|\&gt\;)/is) {
+       return &mt('File does not have [_1] or [_2] starting tag','&lt;html&gt;','&lt;xml&gt;');
+    }
+    if ($filecontents!~/(?:\<|\&lt\;)\/(?:html|xml)(?:\>|\&gt\;)/is) {
+       return &mt('File does not have [_1] or [_2] ending tag','&lt;html&gt;','&lt;xml&gt;');
+    }
+    if ($filecontents!~/(?:\<|\&lt\;)(?:body|frameset)[^\<]*(?:\>|\&gt\;)/is) {
+       return &mt('File does not have [_1] or [_2] starting tag','&lt;body&gt;','&lt;frameset&gt;');
+    }
+    if ($filecontents!~/(?:\<|\&lt\;)\/(?:body|frameset)[^\<]*(?:\>|\&gt\;)/is) {
+       return &mt('File does not have [_1] or [_2] ending tag','&lt;body&gt;','&lt;frameset&gt;');
+    }
+    return '';
+}
 
 sub inserteditinfo {
-      my ($filecontents,$filetype)=@_;
+      my ($filecontents, $filetype, $filename)=@_;
       $filecontents = &HTML::Entities::encode($filecontents,'<>&"');
-#      my $editheader='<a href="#editsection">Edit below</a><hr />';
       my $xml_help = '';
       my $initialize='';
       my $textarea_id = 'filecont';
@@ -1528,38 +1548,45 @@ FULLPAGE
 	  $xml_help=&Apache::loncommon::helpLatexCheatsheet();
       }
 
-      my $cleanbut = '';
-
       my $titledisplay=&display_title();
       my %lt=&Apache::lonlocal::texthash('st' => 'Save and Edit',
 					 'vi' => 'Save and View',
 					 'dv' => 'Discard Edits and View',
 					 'un' => 'undo',
 					 'ed' => 'Edit');
-      my $buttons=(<<BUTTONS);
-$cleanbut
-<input type="submit" name="discardview" accesskey="d"  value="$lt{'dv'}" />
-<input type="submit" name="Undo" accesskey="u"  value="$lt{'un'}" /><hr />
-<input type="submit" name="savethisfile" accesskey="s"  value="$lt{'st'}" />
-<input type="submit" name="viewmode" accesskey="v" value="$lt{'vi'}" />
-BUTTONS
-      $buttons.=&Apache::lonhtmlcommon::spelllink('xmledit','filecont');
+      my $spelllink .=&Apache::lonhtmlcommon::spelllink('xmledit','filecont');
       my $textarea_events = &Apache::edit::element_change_detection();
       my $form_events     = &Apache::edit::form_change_detection();
+      my $htmlerror=&verify_html($filecontents);
+      if ($htmlerror) {
+         $htmlerror='<span class="LC_error">'.$htmlerror.'</span>';
+      }
       my $editfooter=(<<ENDFOOTER);
 $initialize
-<hr />
 <a name="editsection" />
 <form $form_events method="post" name="xmledit">
-$xml_help
-<input type="hidden" name="editmode" value="$lt{'ed'}" />
-$buttons<br />
-<textarea $textarea_events style="width:100%" cols="80" rows="44" name="filecont" id="filecont">$filecontents</textarea>
-<div id="LC_aftertextarea">
-<br />$buttons
-<br />
-$titledisplay
-</div>
+  <div class="LC_edit_problem_editxml_header">
+    <table class="LC_edit_problem_header_title"><tr><td>
+        $filename
+      </td><td align="right">
+        $xml_help
+      </td></tr>
+    </table>
+    <div class="LC_edit_problem_discards">
+      <input type="submit" name="discardview" accesskey="d" value="$lt{'dv'}" />
+      <input type="submit" name="Undo" accesskey="u" value="$lt{'un'}" />
+      $spelllink $htmlerror
+    </div>
+    <div class="LC_edit_problem_saves">
+      <input type="submit" name="savethisfile" accesskey="s" value="$lt{'st'}" />
+      <input type="submit" name="viewmode" accesskey="v" value="$lt{'vi'}" />
+    </div>
+  </div>
+  <textarea $textarea_events style="width:100%" cols="80" rows="44" name="filecont" id="filecont">$filecontents</textarea>
+  <div id="LC_aftertextarea">
+    <br />
+    $titledisplay
+  </div>
 </form>
 </body>
 ENDFOOTER
@@ -1625,6 +1652,9 @@ sub handler {
 	if ($env{'form.savethisfile'} || $env{'form.viewmode'} || $env{'form.Undo'}) {
 	    my $html_file=&Apache::lonnet::getfile($file);
 	    my $error = &Apache::lonhomework::handle_save_or_undo($request, \$html_file, \$env{'form.filecont'});
+            if ($env{'form.savethisfile'}) {
+                $env{'form.editmode'}='Edit'; #force edit mode
+            }
 	}
     }
     my %mystyle;
@@ -1689,11 +1719,12 @@ ENDNOTFOUND
     unless ($env{'request.state'} eq 'published') {
 	if ($env{'form.editmode'} && (!($env{'form.viewmode'})) && (!($env{'form.discardview'})))
 	{
-	    my ($edit_info, $add_to_onload, $add_to_onresize)=
-		&inserteditinfo($filecontents,$filetype);
-
 	    my $displayfile=$request->uri;
 	    $displayfile=~s/^\/[^\/]*//;
+
+	    my ($edit_info, $add_to_onload, $add_to_onresize)=
+		&inserteditinfo($filecontents,$filetype,$displayfile);
+
 	    my %options = 
 		('add_entries' =>
                    {'onresize' => $add_to_onresize,
@@ -1709,9 +1740,7 @@ ENDNOTFOUND
 	    my $start_page = &Apache::loncommon::start_page(undef,$js,
 							    \%options);
 	    $result=$start_page.
-		&Apache::lonxml::message_location().'<h3>'.
-		$displayfile.
-		'</h3>'.
+		&Apache::lonxml::message_location().
 		$edit_info.
 		&Apache::loncommon::end_page();
 	}
@@ -1066,28 +1072,28 @@ Increments the internal counter environm
     $Apache::lonxml::counter += $increment;
 
-    # If the caller supplied the part_id parameter, 
+    # If the caller supplied the response_id parameter, 
     # Maintain its counter.. creating if necessary.
 